
function [EEG, command] = pop_read_mayo(); 
command = '';
EEG = eeg_emptyset;


% get mef directory
[filename, filepath] = uigetfile('*.MEF;*.mef', 'Choose any MEF file -- pop_read_mayo()'); 
drawnow;
if filepath == 0 
	return; 
end

% read EEG (mef) files
disp('Preparing...');
files = dir([filepath '/*.mef']);
n_channels = length(files);
fp = fopen([filepath '/' files(1).name], 'r');
fseek(fp, 16, 'bof');     % number of entries field
n_samples = fread(fp, 1, 'uint64', 0, 'b');
fseek(fp, 64, 'bof');     % sampling frequency field
samp_freq = fread(fp, 1, 'double', 0, 'b');
fclose(fp);

max_comp_len = 0;
for i = 1:length(files)
    if files(i).bytes > max_comp_len
        max_comp_len = files(i).bytes;
    end
end
max_comp_len = max_comp_len - 1024;

ds_samp_freq = 625;
ds_samp_num = floor((n_samples / samp_freq) * ds_samp_freq);
last_ds_samp = ((samp_freq / ds_samp_freq) * (ds_samp_num - 1)) + 1;
ds_times = linspace(1, last_ds_samp, ds_samp_num);
data = zeros(n_channels, ds_samp_num, 'single');
cv_bytes = zeros(4, 1, 'int8');
seg_size = 20000000;    % change this if running into memory problems
if (seg_size > n_samples)
    seg_size = n_samples;
end
channel_data = zeros(seg_size, 1, 'double');
comp_data = zeros(max_comp_len, 1, 'int8');  % reserving this memory, will be cleared on each loop
pack;

[b,a] = cheby1(5, 0.5, ((0.4 * ds_samp_freq) / samp_freq), 'low');  % filter parameters: 5th order, 0.5 dB ripple in passband, min 5 samples per downsampled cycle, lowpass design

n_segs = ceil(n_samples / seg_size);

for i = 1:n_channels
    disp(['Working on channel file "' files(i).name '" (' num2str(i) ' of ' num2str(n_channels) ') ...']);
    fp = fopen([filepath '/' files(i).name], 'r');
    fseek(fp, 24, 'bof');
    chan_name = fread(fp, 32, 'int8');
    j = 1;
    while (chan_name(j));  
        EEG.chanlocs(i).labels(j) = char(chan_name(j));
        j = j + 1;
    end
    comp_data_len = files(i).bytes - 1024;   % get compressed data length
    fseek(fp, 1024, 'bof');
    clear comp_data;
    comp_data = fread(fp, comp_data_len, 'int8');
    fclose(fp);
    j = 1;
    current_val = 0;
    comp_idx = 1;
    ds_samp_start = 1;
    for m = 1:n_segs
        disp(['    Processing segment ' num2str(m) ' of ' num2str(n_segs) '...']);
        seg_start = ((m - 1) * seg_size) + 1;
        if m == n_segs
            seg_end = n_samples;
        else
            seg_end = m * seg_size;
        end
        seg_len = (seg_end - seg_start) + 1;
        disp('        Decompressing...');
        for k = 1:seg_len
            if (comp_data(j) == -128)     % key-sample flag
                j = j + 1;                % this loop assumes the processing machine is big endian
                if (comp_data(j) < 0)     % sign extend
                    cv_bytes(1) = -1;     % 0xFF, 2s complement
                else
                    cv_bytes(1) = 0;
                end
                cv_bytes(2) = comp_data(j);
                j = j + 1;
                cv_bytes(3) = comp_data(j);
                j = j + 1;
                cv_bytes(4) = comp_data(j);
                current_val = typecast(cv_bytes, 'int32');
            else
                current_val = current_val + comp_data(j);
            end
            channel_data(k) = double(current_val);
            j = j + 1; 
        end
        disp('        Filtering...');
        channel_data = filtfilt(b, a, channel_data);
        disp('        Downsampling...');
        ds_samp_end = ds_samp_start;
        while ds_samp_end <= ds_samp_num
            if ds_times(ds_samp_end) <= seg_end
                ds_samp_end = ds_samp_end + 1;
            else
                break;
            end
        end
        ds_samp_end = ds_samp_end - 1;
        for q = ds_samp_start:ds_samp_end   % linear interpolation
        	x = ds_times(q);
        	fx = floor(x);
        	dx = x - fx;
            fx = (fx - seg_start) + 1;
        	fy = channel_data(fx);
        	data(i, q) = single(((channel_data(fx + 1) - fy) * dx) + fy);
    	end
        ds_samp_start = ds_samp_end + 1;
    end
end
clear channel_data comp_data;

% read a time file
disp('Reading times...');
files = dir([filepath '/*.mtf']);
fp = fopen([filepath '/' files(1).name], 'r');
fseek(fp, 16, 'bof');     % number of time entries offset
n_times = fread(fp, 1, 'uint64', 0, 'b');
fseek(fp, 1024, 'bof');
time_arr_len = n_times * 3;
times = fread(fp, time_arr_len, 'uint64', 0, 'b');
fclose(fp);


% read the event file
disp('Reading events...');
files = dir([filepath '/*.mvf']);
fp = fopen([filepath '/' files(1).name], 'r');
fseek(fp, 16, 'bof');     % number of event entries offset
n_events = fread(fp, 1, 'uint64', 0, 'b');
fseek(fp, 1024, 'bof');

time_idx = 1;
n_port_events = 0;
time_arr_len = time_arr_len - 3;
for i = 1:n_events
    type = fread(fp, 4, 'int8', 0, 'b');
    size = fread(fp, 1, 'uint32', 0, 'b');
    if (type(1) == 'P' && type(2) == 'I' && type(3) == '1' && type(4) == 0)
        n_port_events = n_port_events + 1;
        ev_time = fread(fp, 1, 'uint64', 0, 'b');
        ev_val = fread(fp, 1, 'uint64', 0, 'b');
        % find sample number
        while (1)
            if (time_idx <= time_arr_len)
                if (times(time_idx + 3) <= ev_time)
                    time_idx = time_idx + 3;
                else
                    break;
                end
            else
                break;
            end
        end
        time_sample_idx = times(time_idx + 2);
        ev_sample_idx = time_sample_idx + (((ev_time - times(time_idx)) / 1000000) * samp_freq);
        events(n_port_events).type = int32(ev_val);
        events(n_port_events).latency = round((ev_sample_idx * ds_samp_freq) / samp_freq) + 1;
        fseek(fp, size - 24, 'cof');
    else
        fseek(fp, size - 8, 'cof');
    end
end
fclose(fp);


% create eeglab structures
EEG.filepath = filepath; 
EEG.data = data;
EEG.srate = ds_samp_freq;
EEG.setname = '';
EEG.pnts = ds_samp_num;
EEG.xmin = 0;
EEG.xmax = (ds_samp_num - 1) / ds_samp_freq;
EEG.nbchan = n_channels;
EEG.trials = 1;
EEG.event = events;
EEG.times = ds_times;
EEG = eeg_checkset(EEG,'eventconsistency');
EEG = eeg_checkset(EEG,'makeur');

command = 'EEG = pop_read_mayo();';

return;


